package net.sf.sql2java.common.parameters;

import java.util.HashMap;
import java.util.Map;

/**
 * A Parameterized String
 * 
 * Ever wanted to embed parameters inside a String? Well then you might be glad to 
 * use the ParameterizedString. This class will give you the opportunity to use parameters 
 * inside a String. After parsing the string, the provided parameters will be replaced
 * with their values, leaving a perfect String.
 * <p>
 * Lets suggest our source string is <code>Username: :{un}</code><br>
 * We'll also supply '<code>Lewis</code>' as the value for the '<code>un</code>' 
 * parameter as we want to replace the parameter by the username.<br>
 * <br>
 * To use this class, we'll have to supply our source string at construction time.<br>
 * <code>
 * ParameterizedString ps = new ParameterizedString(sourceString);
 * ps.addParameter("un", "Lewis");
 * System.out.println("result = " + ps.parse());
 * </code>
 * 
 * @author Daan Gerits
 */
public class ParameterizedString {
    protected String content;
    private Map<String, Parameter> parameters;

// -- Constructors ------------------------------------------------------------
    
    public ParameterizedString(String content) {
        this.content = content;
        
        this.parameters = new HashMap<String, Parameter>();
    }

// -- Class Conversions -------------------------------------------------------
    
    public String toString() {
        return parse();
    }
    
// -- Parsing -----------------------------------------------------------------
    
    public String parse() {
        // -- create the result object
        String result = content;
        
        // -- run trough the properties and try to match
        for (Parameter parameter : parameters.values()) {
            result = parameter.replace(result);
        }
        
        // -- let the modifier go ...
        result = new Modifier().replace(result);
        
        // -- return the result object
        return result;
    }
    
// -- Parameter Conversion ----------------------------------------------------
/*
    
    private String removeDoubles(String input) {
        input = input.replaceAll("\\.\\.", ".");
        input = input.replaceAll("//", "/");
        
        return input;
    }
    
/**/
// -- Parameter Management ----------------------------------------------------
    
    public void addParameter(Parameter parameter) {
        parameters.put(parameter.getName(), parameter);
    }
    
    public void addParameter(String name, String value) {
        addParameter(new Parameter(name, value));
    }
    
    public void removeParameter(Parameter parameter) {
        parameters.remove(parameter.getName());
    }
}
