package net.sf.sql2java.fetching.sql;

import java.util.ArrayList;
import java.util.List;

import net.sf.sql2java.common.beans.DatabaseConnection;
import net.sf.sql2java.common.beans.SourceFilter;
import net.sf.sql2java.common.configuration.Configurer;

import org.apache.commons.configuration.ConfigurationException;
import org.apache.log4j.Logger;

public class SQLFetcherConfigurer extends Configurer<SQLFetcher> {
	private static final Logger LOG = Logger.getLogger(SQLFetcherConfigurer.class);

// -- Configuration -----------------------------------------------------------
	
	/**
	 * Configure the given <code>SQLFetcher</code>
	 * <p>
	 * We'll provide the given fetcher with a <code>DatabaseConnection</code> 
	 * and a <code>SourceFilter</code>. Together with the types list, they will
	 * be used to determine which table, views, ... will be used during
	 * generation.
	 * 
	 * @param 	fetcher		The <code>SQLFetcher</code> we need to configure.
	 * @throws	ConfigurationException	Is thrown when this Configurer has no
	 * 									configuration set.
	 * 
	 * @see	SQLFetcher
	 * @see ConfigurationException
	 */
	public void configure(SQLFetcher fetcher) throws ConfigurationException {
		LOG.info("Configuring SQLFetcher.");
		
		// -- check if the configuration is valid
		if (getConfiguration() == null)
			throw new ConfigurationException("No configuration has been set");
		
		// -- create the database connection
		DatabaseConnection connection = createDatabaseConnection();
		fetcher.setConnection(connection);
		LOG.info("Created DatabaseConnection to " + connection.getUrl());
		
		// -- create the filter
		SourceFilter filter = createFilter();
		fetcher.setFilter(filter);
		
		// -- create the types
		List<String> types = getTypes();
		fetcher.setTypes(types);
		
		String typeString = ""; 
		for (String type : types) typeString += (type + " "); 
		LOG.info("Using following types while fetching: " + typeString);
	}
	
// -- Helpers -----------------------------------------------------------------
	
	/**
	 * Create a new DatabaseConnection Object.
	 * <p>
	 * Use the values from the configuration to create a new DatabaseObject.
	 * The following values should have been set:
	 * <ul>
	 * <li>fetcher.database.driver
	 * <li>fetcher.database.url
	 * <li>fetcher.database.username
	 * <li>fetcher.database.password
	 * </ul>
	 * 
	 * @return	This method returns a DatabaseConnection holding the values
	 * 			found inside the configuration.
	 * 
	 * @see DatabaseConnection
	 */
	protected DatabaseConnection createDatabaseConnection() {
		// -- create the result
		DatabaseConnection result = new DatabaseConnection();
		
		result.setDriverClass(getConfiguration().getString("fetcher.database.driver"));
		result.setUrl(getConfiguration().getString("fetcher.database.url"));
		result.setUsername(getConfiguration().getString("fetcher.database.username"));
		result.setPassword(getConfiguration().getString("fetcher.database.password"));
		
		// -- return the result
		return result;
	}
	
	/**
	 * Create a new SourceFilter.
	 * <p>
	 * Create a <code>SourceFilter</code> using the parameters found inside
	 * the configuration.
	 * 
	 * @return	A new SourceFilter, constructed using the values found inside
	 * 			the configuration.
	 * 
	 * @see SourceFilter
	 */
	protected SourceFilter createFilter() {
		SourceFilter result = null;
		
		return result;
	}
	
	/**
	 * Get the types we want to use during fetching.
	 * 
	 * @return 	This method returns a list of String's, containing the types to
	 * 			look for while fetching. 
	 */
	@SuppressWarnings("unchecked")
	protected List<String> getTypes() {
		List<String> result = new ArrayList<String>();
		List<String> content = getConfiguration().getList("fetcher.types.type");
		
		// -- add all values
		for (String str : content)
			result.add(str.toUpperCase());
		
		// -- return the result
		return result;
	}
}
